<?php

/*
 Plugin Name: Table of Contents Creator
 Plugin URI: http://markbeljaars.com/plugins/TOCC-plugin/
 Description: Table of Contents Creator (TOCC) automatically generates a dynamic site wide table of contents that is always up-to-date. All entries are navigable making your site very SEO friendly. TOCC can be configured to display static pages, blog entries and forum comments. Another great feature of TOCC is the ability to include anchor tags on any page marked with a special class. This feature allows links to articles, downloads or even other sites to appear within the table of contents as if they are part of your site's navigation. To generate a table of contents, simply include the <code>&lt;!-- toc-creator --&gt;</code> tag on any page, or use the handy page creation feature located on the plugin admin page.
 Version: 1.4.4
 Author: Mark Beljaars
 Author URI: http://markbeljaars.com
 */

/*
 Copyright (C) 2009 Mark Beljaars, markbeljaars.com
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published 
 by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see http://www.gnu.org/licenses/.
 */

define('TOCC_VERSION', '1.4.4');
define('TOCC_PLUGIN_PATH', 'wp-content/plugins/' . basename(dirname(__FILE__)));
define('TOCC_TAG', '<!-- toc-creator -->');
define('TOCC_CSS_FILE', 'tocc.css');
define('TOCC_NUMBER_SENTINEL', '{x}');
define('TOCC_ICON_NONE', 'None');
define('TOCC_ICON_THEMES', constant('TOCC_ICON_NONE') . ',Professional,Square,Bling,Blue,Handdrawn');
define('TOCC_ALLPOSTS_PAGEID', 'all_posts');
define('TOCC_ALLCOMMENTS_PAGEID', 'all_comments');

add_action('plugins_loaded','tocc_loaded_hook');
add_action('init', 'tocc_init_hook');
add_action('wp_head', 'tocc_head_hook');
add_filter('the_content', 'tocc_display_hook');

if ( is_admin() ) {
	add_action('admin_menu', 'tocc_menu_hook');
	add_filter('plugin_action_links','tocc_filter_settings_option', 10, 2);
	register_uninstall_hook(__FILE__, 'tocc_uninstall_hook');
}

function tocc_loaded_hook() {

    load_plugin_textdomain('tocc', constant('TOCC_PLUGIN_PATH'), basename(dirname(__FILE__)));
    tocc_check_upgrade();

    global $tocc_options;
    $tocc_options = array(
       'toc_title' => '<h1>' . __('Table of Contents', 'tocc') . '</h1>',
	    'date_format' => __('F j, Y, H:i', 'tocc'),
	    'more_prompt' => '<strong>' . __('More...', 'tocc') . '</strong>',
	    'page_title' => '<h2>' . __('Static Pages', 'tocc') . '</h2>',
	    'blog_warntext' => sprintf(__('(last %s posts shown)', 'tocc'), constant('TOCC_NUMBER_SENTINEL')),
	    'blog_authorprefix' => __('by', 'tocc') . ':',
	    'blog_title' => '<h2>' . __('Blog Posts', 'tocc') . '</h2>',
	    'forum_title' => '<h2>' . __('Forum Comments', 'tocc') . '</h2>',
	    'forum_page' => __('Forum', 'tocc'),
	    'forum_warntext' => sprintf(__('(last %s comments shown)', 'tocc'), constant('TOCC_NUMBER_SENTINEL')),
	    'forum_authorprefix' => __('by', 'tocc') . ':',
	    'icon_set' => constant('TOCC_ICON_NONE'),
	    'separate_lists' => '',
	    'all_listitems' => '',
	    'show_ack' => 'checked',
	    'show_static' => 'checked',
	    'page_depth' => 1,
	    'link_class' => 'tocc',
	    'page_exclude' => array (),
	    'show_blog' => 'checked',
	    'blog_warn' => 'checked',
	    'blog_author' => 'checked',
	    'blog_date' => 'checked',
	    'blog_count' => -1,
	    'blog_more' => 'checked',
	    'blog_info' => 'checked',
	    'blog_exclude' => array (),
	    'show_forum' => '',
	    'forum_warn' => 'checked',
	    'forum_author' => 'checked',
	    'forum_date' => 'checked',
	    'forum_count' => 10,
	    'forum_more' => 'checked',
	    'toc_page' => __('Site Map', 'tocc'),
	    'toc_slug' => __('site-map', 'tocc')
	    );

    foreach($tocc_options as $option => $value) {
	    add_option('tocc_' . $option, $value);
    }
}

function tocc_uninstall_hook() {

	tocc_loaded_hook();
	foreach(array_keys($GLOBALS['tocc_options']) as $option) {
		delete_option('tocc_' . $option);
	}

	delete_option('tocc_debug_mode');
	delete_option('tocc_version');
}

function tocc_init_hook() {
    load_plugin_textdomain('tocc', constant('TOCC_PLUGIN_PATH'), basename(dirname(__FILE__)));
}

function tocc_head_hook() {
	echo '<link rel="stylesheet" type="text/css" media="screen" href="' . get_bloginfo('wpurl') . '/'
		. constant('TOCC_PLUGIN_PATH') . '/' . constant('TOCC_CSS_FILE') . '" />' . PHP_EOL;
}

function tocc_menu_hook() {
	add_options_page('Table of Contents Creator', 'Table of Contents Creator', 8, __FILE__
			, 'tocc_plugin_options');
}

function tocc_display_hook($content = '') {

	if (strpos($content, constant('TOCC_TAG')) === false)
		return $content;

	$pages = get_pages('depth=' . get_option('tocc_page_depth') . '&sort_column=menu_order');
	$icon_set = get_option('tocc_icon_set') == constant('TOCC_ICON_NONE') ? '' : get_option('tocc_icon_set');
	$link_class = get_option('tocc_link_class');
	$as_list_items = $icon_set || get_option('tocc_all_listitems');
	$page_for_posts = get_option('page_for_posts');
	$separate_lists = get_option('tocc_separate_lists');
	$forum_page = get_option('tocc_forum_page');
	$exluded_pages = get_option('tocc_page_exclude');
	$show_static = get_option('tocc_show_static');
	$show_blog = get_option('tocc_show_blog');
	$show_forum = get_option('tocc_show_forum');

	$tocc = PHP_EOL . PHP_EOL . '<!-- CREATED BY "TABLE OF CONTENTS GENERATOR v' . constant('TOCC_VERSION') . '" BY MARK BELJAARS -->' 
			. PHP_EOL; 

	if(get_option('tocc_debug_mode')) {
		$tocc .= '<!-- TOCC DEBUG INFORMATION FOLLOWS...' . PHP_EOL;
		foreach($GLOBALS['tocc_options'] AS $option => $value) {
			$tocc .= '   ' . $option . ' => ' . get_option('tocc_' . $option) . ' (' . $value . ')' . PHP_EOL;
			if(is_array(get_option('tocc_' . $option))) {
				foreach(get_option('tocc_' . $option) as $key => $value) {
					$tocc .= '      ' . $key . ' => ' . $value . PHP_EOL;
				}
			}
		}
		$tocc .= '-->' . PHP_EOL;
	}
    
	$tocc .= PHP_EOL . '<div id="tocc">';
	if(get_option('tocc_toc_title')) {
		$tocc .= get_option('tocc_toc_title') . PHP_EOL;
	}

	if(isset($_GET['page'])) {
	
		if($_GET['page'] == constant('TOCC_ALLPOSTS_PAGEID')) {
			$blog_title = get_option('tocc_blog_title');
			$tocc .= $blog_title . PHP_EOL
					. tocc_display_posts('', '', 0, -1, $icon_set, $as_list_items);
		}
		
		elseif($_GET['page'] == constant('TOCC_ALLCOMMENTS_PAGEID')) {
			$tocc .= get_option('tocc_forum_title') . PHP_EOL
					. tocc_display_forum('', 0, -1, $icon_set);
		}
	}

	else {

		if($separate_lists && $show_static) {
			$tocc .= get_option('tocc_page_title') . PHP_EOL;
		}

		if($show_static) {

			$lastpageID = "";
			$lastparentID = "";
			$depth = 0;
			foreach ($pages as $pagg) {

				if ($pagg->post_parent == 0) {
					$depth = 0;
					$lastparentID = $pagg->post_parent;
				}
				elseif ($pagg->post_parent == $lastpageID) {
					$depth++;
					$lastparentID = $pagg->post_parent;
				}
				elseif ($pagg->post_parent != $lastpageID && $pagg->post_parent != $lastparentID) {
					$depth--;
					$lastparentID = $pagg->post_parent;
				}

				if (!($exluded_pages && in_array($pagg->post_title, $exluded_pages))
							&& (!$show_blog || $pagg->ID != $page_for_posts)
							&& (!$show_forum || $pagg->post_title != $forum_page || !function_exists('latest_activity'))) {
					$tocc .= tocc_display_static($pagg, $depth, $icon_set, $link_class, $as_list_items);
				}

				if ($show_blog && !$separate_lists && $pagg->ID == $page_for_posts) {
					$tocc .= tocc_display_posts($pagg->post_title, get_permalink($pagg->ID), $depth, get_option('tocc_blog_count')
							, $icon_set, $as_list_items);
				}

				if ($show_forum && !$separate_lists && $pagg->post_title == $forum_page && function_exists('latest_activity')) {
					$tocc .= tocc_display_forum($pagg, $depth, get_option('tocc_forum_count'), $icon_set);
				}

				$lastpageID = $pagg->ID;
			}
		}
	
		if ($show_blog && (!$show_static || !$page_for_posts || $separate_lists)) {
			$blog_title = get_option('tocc_blog_title');
			$blog_page = $page_for_posts ? $page_for_posts : get_option('home');
			if($separate_lists) {
				$tocc .= PHP_EOL . PHP_EOL . ($show_static ? '<br />' : '') . $blog_title . PHP_EOL;			
				$blog_title = '';
			}
			else {
				$blog_title = strip_tags(get_option('tocc_blog_title'));
			}
			$tocc .= tocc_display_posts($blog_title, $blog_page, 0, get_option('tocc_blog_count'), $icon_set, $as_list_items);
		}
	
		if ($show_forum && (!$show_static || $separate_lists) && function_exists('latest_activity')) {
			if($separate_lists) {
				$tocc .= PHP_EOL . PHP_EOL . (($show_static || $show_blog) ? '<br />' : ''). get_option('tocc_forum_title') . PHP_EOL;
				$forum_page = '';
			}
			$tocc .= tocc_display_forum($forum_page, 0, get_option('tocc_forum_count'), $icon_set);
		}
	}
	
	if (get_option('tocc_show_ack')) {
		$tocc .= '<br /><br />' . PHP_EOL . PHP_EOL
				. '<div id="tocc_footer"><p>' . __('Generated by', 'tocc') . ' Table of Contents Creator v' . constant('TOCC_VERSION') 
				. '<br />' . __('by', 'tocc') . ' <a href="http://markbeljaars.com/">Mark Beljaars</a></p></div>'. PHP_EOL;
	}

	$content .= '</div>';
	$content = str_replace(constant('TOCC_TAG'), $tocc, $content);
	return $content;
}

function tocc_display_static($pagg, $depth, $icon_set, $link_class, $as_list_items) {

	$content .= '<div class="tocc_depth' . $depth 
				. ($icon_set ? ' tocc_icon ' . $icon_set . '_page' : '') 
				. '">'
				. ($as_list_items ? '<ul><li>' : '') 
				. '<a href="' . get_permalink($pagg->ID) . '">' . $pagg->post_title . '</a>'
				. ($as_list_items ? '&nbsp;</li></ul>' : '')
				. '</div>' . PHP_EOL;

	if($link_class) {
		preg_match_all('#<a\s+
							(?:(?= [^>]* href\s*=\s*["\']   (?P<href>  [^"\']*) ")|)
							(?:(?= [^>]* class\s*=\s*["\']  (?P<class> [^"\']*) ")|)
							[^>]*>
							(?P<text>[^<]*)
							</a>
							#xi', $pagg->post_content, $matches, PREG_SET_ORDER);

		$list_init = 0;
		if($matches) {
			foreach ($matches as $match) {
				if ($match['class'] == $link_class) {
					if(!$list_init) {
						$content .= '<div class="tocc_depth' . ($depth +1) 
									. ($icon_set ? ' tocc_icon ' . $icon_set . '_page_link' : '') 
									. '">'
									. '<ul>';
						$list_init = 1;
					}
					$content .= '<li><a href="' . $match['href'] . '">' . $match['text'] . '</a>&nbsp;</li>';
				}
			}
			$content .= $list_init ? ('</ul></div>' . PHP_EOL) : '' ;
		}
	}

	return $content;
}

function tocc_display_posts($pagename, $permalink, $depth, $blog_count, $icon_set, $as_list_items) {

	global $wpdb;

	if($pagename) {
		$content .= '<div class="tocc_depth' . $depth  
			. ($icon_set ? ' tocc_icon ' . $icon_set . '_page' : '')
			. '">'
			. ($as_list_items ? '<ul><li>' : '') 
			. '<a href="' . $permalink . '">' . $pagename . '</a>' 
			. (get_option('tocc_blog_warn') && $blog_count > 0 ? ' ' . str_replace(constant('TOCC_NUMBER_SENTINEL')
				, $blog_count, get_option('tocc_blog_warntext')) : '')
			. ($as_list_items ? '&nbsp;</li></ul>' : '')
			. '</div>' . PHP_EOL;
		$depth++;
	}
	
	$blog_exclude = get_option('tocc_blog_exclude');
	$blog_info = get_option('tocc_blog_info');
	$show_author = get_option('tocc_blog_author');
	$show_date = get_option('tocc_blog_date');
	$author_prefix = get_option('tocc_blog_authorprefix');
	$date_format = get_option('tocc_date_format');

	$categories = get_categories();
	foreach ($categories as $cat) {
		
		if (!($blog_exclude && in_array($cat->cat_name, $blog_exclude))) {

			$querystr = "
				SELECT wpposts.ID, wpposts.post_title, wpposts.post_date, wpposts.post_author
				FROM $wpdb->terms wpterms
					INNER JOIN $wpdb->term_taxonomy wptax on wpterms.term_id = wptax.term_id
					INNER JOIN $wpdb->term_relationships wprel on wprel.term_taxonomy_id = wptax.term_taxonomy_id
					INNER JOIN $wpdb->posts wpposts on wpposts.ID = wprel.object_id
				WHERE wptax.taxonomy= 'category' 
					AND wpposts.post_type = 'post'
					AND wpposts.post_status = 'publish'
					AND wpterms.term_id IN (" . $cat->cat_ID . ")
				ORDER BY post_date_gmt DESC"
					. ($blog_count > 0 ? " LIMIT 0," . $blog_count : "") . ";";
			$posts = $wpdb->get_results($querystr, OBJECT);

			$usequery = (count($posts) > 0);
			if(!$usequery) {
				$posts = get_posts('orderby=date&order=DESC&numberposts=' . $blog_count . '&cat=' . $cat->cat_ID);
				if(count($posts) > 0) {
					$content .= PHP_EOL . '<!-- (debug) Post extraction resorted to get_post() function -->' . PHP_EOL;
				}
			}

			$count = $cat->count;
			if($blog_count <= 0 || $count < $blog_count) {
				$count = count($posts);
			}
			
			$content .= '<div class="tocc_depth' . ($depth) 
						. ($icon_set ? ' tocc_icon ' . $icon_set . '_blog_cat' : '') 
						. '">'
						. ($as_list_items ? '<ul><li>' : '') 
						. '<a href="' . get_category_link($cat->cat_ID) . '">' . $cat->cat_name . '</a>' 
						. ($posts && $blog_info	? ' (' . $count . ')' : '') 
						. ($as_list_items ? '&nbsp;</li></ul>' : '')
						. '</div>' . PHP_EOL;

			if($posts) {
			
				$content .= '<div class="tocc_depth' . ($depth +1) 
							. ($icon_set ? ' tocc_icon ' . $icon_set . '_blog_post' : '') 
							. '">'
							. '<ul>';

				foreach ($posts as $post) {
					$user = get_userdata($post->post_author);
					$content .= '<li>' 
								. '<a href="' . get_permalink($post->ID) . '">' . $post->post_title . '</a>&nbsp;'
								. ($show_author ?  $author_prefix . ' <a href="' 
									. $user->user_url . '">' . $user->display_name . '</a>' : '') 
								. ($show_date ? '<br /><small>' . date($date_format, strtotime($post->post_date)) . '</small>' : '') 
								. '</li>' . PHP_EOL;
				}
				$content .= '</ul>' 
							. '</div>' . PHP_EOL;
			}
		}
	}
	
	if (get_option('tocc_blog_more') && get_option('tocc_more_prompt') && $blog_count > 0) {
		$content .= '<div class="tocc_depth' . $depth . '">' 
					. '<a href="' . tocc_strleft(tocc_selfURL(), '?') . '?page=' . constant('TOCC_ALLPOSTS_PAGEID') . '">' 
					. get_option('tocc_more_prompt') 
					. '</a></div>';
	}
	
	return $content;
}

function tocc_display_forum($pagg, $depth, $forum_count, $icon_set) {
	
	global $wpdb;

	ob_start();
	latest_activity($forum_count > 0 ? $forum_count : 100000);
	$posts = ob_get_contents();
	ob_end_clean();
	
	$content .= '<div class="tocc_depth' . $depth 
				. ($icon_set ? ' tocc_icon ' . $icon_set . '_page' : '') 
				. '">' 
				. ($as_list_items ? '<ul><li>' : '') 
				. '<a href="' . get_permalink($pagg->ID) . '">' . $pagg->post_title . '</a>'
				. (get_option('tocc_forum_warn') && $forum_count > 0 ? ' ' . str_replace(constant('TOCC_NUMBER_SENTINEL')
					, $forum_count, get_option('tocc_forum_warntext')) : '')
				. ($as_list_items ? '&nbsp;</li></ul>' : '')
				. '</div>' . PHP_EOL
				. '<div class="tocc_depth' . ($depth +1)
				. ($icon_set ? ' tocc_icon ' . $icon_set . '_forum' : '')
				. '">'
				. '<ul>';

	preg_match_all('#<li>(.*?)</li>#xi', $posts, $matches, PREG_SET_ORDER);
	
	$show_author = get_option('tocc_forum_author');
	$show_date = get_option('tocc_forum_date');
	$author_prefix = get_option('tocc_forum_authorprefix');
	$date_format = get_option('tocc_date_format');
	
	foreach ($matches as $match) {
		
		preg_match_all('#(.*?)</a>#xi', $match[1], $comment, PREG_SET_ORDER);
		preg_match_all('#</a>\s*by:\s*(.*?)<br\s*/>#xi',$match[1], $author, PREG_SET_ORDER);
		preg_match_all('#<small>(.*?)</small>#xi', $match[1], $date, PREG_SET_ORDER);
		
		$content .= '<li>'
					. $comment[0][0]
					. ($show_author ? ' ' . $author_prefix . ' ' . $author[0][1] : '')
					. ($show_date ? '<br /><small>' .  date($date_format, strtotime($date[0][1])) 
					. '</small>' : '')
					. '&nbsp;</li>';
	}

	$content .= '</ul>' 
				. '</div>' . PHP_EOL;

	if (get_option('tocc_forum_more') && get_option('tocc_more_prompt') && $forum_count > 0) {
		$content .= '<div class="tocc_depth' . ($depth + 1) . '">'
					. '<a href="' . tocc_strleft(tocc_selfURL(), '?') . '?page=' . constant('TOCC_ALLCOMMENTS_PAGEID') . '">' 
					. get_option('tocc_more_prompt') 
					. '</a></div>';
	}
				
	return $content;
}


function tocc_check_upgrade() {

    if(get_option('tocc_version')) {
    
	    // retrieve the version prior to update and convert into a number
	    $old_ver = explode('.', get_option('tocc_version') . '.0.0.0');
	    $old_ver = $old_ver[0] * 1000000 + $old_ver[1] * 1000 + $old_ver[2];
	
	    // manipulate options to ensure the latest version site-map is visually compatible with previous versions
	    if(get_option('tocc_version') != constant('TOCC_VERSION')) {
		
		    if($old_ver < 1003000) {

			    // add header tags to title - prior to version 1.3 the header tags were hard coded
			    $title = get_option('tocc_toc_title');
			    if($title && substr($title, 0, 1) != '<') {
				    update_option('tocc_toc_title', '<h1>' . $title . '</h1>');
			    }
		    }
		
		    if($old_ver < 1004000) {
		
			    // set icon theme to handrawn - versions prior to 1.4 had a show icons setting and the only icon theme was "handdrawn"
			    if(get_option('tocc_show_icons')) {
				    update_option('tocc_icon_set', 'Handdrawn');
			    delete_option('tocc_show_icons');
			    }

			    // add formatting tags to blog title - formatting tags were not required prior to version 1.4 
			    $title = get_option('tocc_blog_title');
			    if($title && substr($title, 0, 1) != '<') {
				    update_option('tocc_blog_title', '<h2>' . $title . '</h2>');
			    }

		    }
		
		    update_option('tocc_version',constant('TOCC_VERSION'));	
	    }
    }
    
    else {
        update_option('tocc_version',constant('TOCC_VERSION'));	
    }
}

// The following function by S.M. Saidur Rahman
// http://ranawd.wordpress.com/2008/11/18/how-to-get-the-current-page-url/
function tocc_selfURL() {
	$s = empty($_SERVER['HTTPS']) ? '' : ($_SERVER["HTTPS"] == 'on') ? 's' : '';	
	$protocol = tocc_strleft(strtolower($_SERVER['SERVER_PROTOCOL']), '/').$s;
	$port = ($_SERVER['SERVER_PORT'] == '80') ? '' : (':'.$_SERVER['SERVER_PORT']);
	return $protocol.'://'.$_SERVER['SERVER_NAME'].$port.$_SERVER['REQUEST_URI'];
}

function tocc_strleft($s1, $s2) {
	$values = substr($s1, 0, strpos($s1, $s2));
	return  $values;
}
// ------------------------------------------------------------------------------

function tocc_filter_settings_option( $links, $file ) {
	static $this_plugin;
	
	if (!$this_plugin)
		$this_plugin = plugin_basename(__FILE__);

	if ($file == $this_plugin) {
		$settings_link = '<a href="options-general.php?page=table-of-contents-creator/table_of_contents_creator.php">' 
							. __('Settings', 'tocc') . '</a>';
		array_unshift($links, $settings_link);
	}
	return $links;
}

function tocc_option($option_text, $tooltip_text, $option_name, $element_type, $other_option = '', $disabled = 0, 
			$element_index = 'post_title') {

	$help_link = '<span title="' . __('Click for Help!', 'tocc') . '" '
				  . 'onclick="toggleVisibility(\'' . $option_name . '_tip\');" ' 
				  . 'onmouseover="this.style.textDecoration = \'underline\'" ' 
				  . 'onmouseout="this.style.textDecoration = \'none\'" ' 
				  . 'style="cursor:pointer;"'
				  . '>';
						
	$inner_border = 'border-color:#CEE1EF; border-style:solid; border-width:1px; height:auto; margin:5px 0px 5px 0px; overflow:auto;'
					  . 'padding:0.5em 0.5em;-moz-border-radius: 0 0 3px 3px;-webkit-border-bottom-left-radius: 3px;'
					  . '-webkit-border-bottom-right-radius: 3px;-khtml-border-bottom-left-radius: 3px;' 
					  . '-khtml-border-bottom-right-radius: 3px;	border-bottom-left-radius: 3px;border-bottom-right-radius: 3px;';

	$label = '<label for="' . $option_name . '" class="selectit" ' 
			 . 'style="margin-left:' . ($element_type != 'checkbox' ? '2' : '0') . 'px;' . ($disabled ? 'color: grey;' : '') . '"'
			 . '>';

	echo '<p class="meta-options">';

	switch ($element_type) {

		case 'text' :

			echo $label 
				. ($tooltip_text ? $help_link : '') . $option_text .  ($tooltip_text ? '</span>' : '') . ':&nbsp;&nbsp;'
				. '</label>'
				. '<input name="' . $option_name . '" type="text" size="' . $other_option . '" ' 
				. 'value="' . htmlspecialchars(get_option($option_name)) . '"' 
				. ($disabled ? ' disabled="true"' : '') 
				. ' />';

			break;

		case 'select' :

			echo $label
				. ($tooltip_text ? $help_link : '')	. $option_text .  ($tooltip_text ? '</span>' : '') . ':&nbsp;&nbsp;'
				. '</label>'
				. '<select name="' . $option_name . '"' . ($disabled ? ' disabled="true"' : '') . '>';
				
			foreach ($other_option as $opts) {

				$value = ($element_index ? $opts-> {$element_index} : $opts);
				echo '<option value="' . $value . '"' 
					. (htmlspecialchars(get_option($option_name)) == $value ? ' selected="selected"' : '') 
					. '>' 
					. $value . '</option>';
			}
			
			echo '</select>';			
				
			break;

		case 'checkbox' :

			if (!is_array($other_option)) {

				echo '<input name="' . $option_name . '" type="checkbox" id="'. $option_name .'"' 
					. (get_option($option_name) ? ' checked="checked"' : '') 
					.  ($disabled ? ' disabled="true"' : '') . ' />'
					. '&nbsp;&nbsp;'
					. ($disabled ? $label : '')
					. ($tooltip_text ? $help_link : '')	. $option_text 
					. ($disabled ? '</label>' : '')
					. ($tooltip_text ? '</span>' : '');
			}

			else {

				echo $label . '<strong>' 
					. ($tooltip_text ? $help_link : '')	. $option_text . ':' .  ($tooltip_text ? '</span>' : '')
					. '</strong>' . '</label>' . PHP_EOL
					. '<div style="' . $inner_border . '">';

				foreach ($other_option as $opts) {

					echo '<input type="checkbox" name="' . $option_name . '[]" value="' . $opts-> {$element_index} . '"' 
						. (get_option($option_name) && in_array($opts-> {$element_index}, get_option($option_name)) ? ' checked="checked"' : '') 
						. ($disabled ? ' disabled="true"' : '') . '>'  . PHP_EOL 
						. '&nbsp;&nbsp;'
						. $opts-> {$element_index}
						. '<br /></input>';		
				}
				
				echo '</div>';
			}
			
			break;
			
	}

	echo '</p>';

	if($tooltip_text) {
		
		echo '<div id="' . $option_name . '_tip" style="display:none;margin-left:8px;color:#21759b;">'
			. $tooltip_text
			. '<br />&nbsp;<br /></div>' . PHP_EOL;
	}
}

function tocc_option_header($heading_text, $heading_name, $is_hidden='true') {

	if($_POST[$heading_name . '_display']) {
		$is_hidden = $_POST[$heading_name . '_display'] == 'none';
	}
	
	echo '<input type="hidden" id="' . $heading_name . '_display" name="' 
		. $heading_name . '_display" value="' . $_POST[$heading_name . '_display'] . '">' . PHP_EOL
		. '<div id="normal-sortables" class="meta-box-sortables">'
		. '<div class="postbox">'
		. '<div class="handlediv" title="' . __('Click to display or hide section options.', 'tocc') 
		. '" onclick="toggleVisibility(\'' . $heading_name . '\');">'
		. '<br /></div><h3 class="hndle" title="' . __('Click to display or hide section options.', 'tocc') 
		. '" onclick="toggleVisibility(\'' . $heading_name . '\');" ><span>' . $heading_text . '</span></h3>'
		. '<div class="inside" id="' . $heading_name . '" style="display:' . ($is_hidden ? 'none' : 'block') . ';">' . PHP_EOL;
}

function tocc_option_end() {
	echo '</div></div></div>' . PHP_EOL;
}

function tocc_plugin_options() {
		
	echo '<script type="text/javascript">' . PHP_EOL 
		. '<!--' . PHP_EOL 
		. 'function toggleVisibility(id) { ' . PHP_EOL 
		. '   var e = document.getElementById(id);' . PHP_EOL 
		. '   if(e.style.display == "block")' . PHP_EOL 
		. '      e.style.display = "none";' . PHP_EOL 
		. '   else' . PHP_EOL 
		. '      e.style.display = "block";' . PHP_EOL 
		. '	document.getElementById(id.concat("_display")).value = e.style.display;' . PHP_EOL
		. '}' . PHP_EOL 
		. '-->' . PHP_EOL
		. '</script>' . PHP_EOL 
		. PHP_EOL;		
	
	if (isset ($_POST['set_defaults'])) {

		foreach($GLOBALS['tocc_options'] as $option => $value) {
			update_option('tocc_' . $option, $value);
		}

		echo '<div id="message" class="updated fade"><p><strong>' . __('Default options loaded!', 'tocc') . '</strong></p></div>';
	} 

	elseif (isset ($_POST['create_page'])) {

		$title = stripslashes((string) $_POST['tocc_toc_page']);
		$slug = stripslashes((string) $_POST['tocc_toc_slug']);

		update_option('tocc_toc_page', $title);
		update_option('tocc_toc_slug', $slug);

		$postID = wp_insert_post(array (
			'post_title' => $title,
			'post_content' => constant('TOCC_TAG'),
			'post_status' => 'publish',
			'post_author' => 1,
			'post_name' => $slug,
			'post_type' => 'page'
		));

		echo '<div id="message" class="updated fade"><p><strong>';

		if($postID) {
			echo '(ID ' . $postID . ')' . __('TOC page was successfully created.', 'tocc');
		} 
		else {
			echo __('TOC page could not be created!', 'tocc');
		}

		echo '</strong></p></div>';
	} 

	elseif(isset ($_POST['info_update'])) {

		if($_POST['tocc_toc_title'] == 'DEBUG_ON') {
			update_option('tocc_debug_mode', 'checked');
		}

		elseif($_POST['tocc_toc_title'] == 'DEBUG_OFF') {
			update_option('tocc_debug_mode', '');
		}

		else {
			
			foreach(array_keys($GLOBALS['tocc_options']) as $option) {
			
				update_option('tocc_' . $option, 
					!is_array($_POST['tocc_' . $option])
						? stripslashes($_POST['tocc_' . $option]) 
						: $_POST['tocc_' . $option]
				);
			}
		}

		echo '<div id="message" class="updated fade"><p><strong>' . __('Configuration updated!', 'tocc') . '</strong></p>';

		if (!get_option('tocc_show_ack')) {
			echo '<p>' . __(
'The author of this software has spent uncountable hours of his own time to develop and test this plugin only to offer it free to you. Please consider leaving the author acknowledgment enabled as a small token of your appreciation.'
				, 'tocc')
				. '</p>';
		}

		echo '</div>';
	}

	echo '<div class="wrap"><h2>Table of Contents Creator v' . constant('TOCC_VERSION') . '</h2>' . PHP_EOL 
		. '<div id="poststuff" class="metabox-holder">' . PHP_EOL
		. '<form method="post" action="" name="dofollow">' . PHP_EOL
		. '<p>' . __(
'The TOC Creator Plugin for Wordpress automatically generates a site wide table of content for your blog. The  TOC will always remain up date and is a SEO freindly way of linking all pages on your site.'
		, 'tocc') . '</p>' . PHP_EOL . '<p>'; 
		
	printf( __(
'To use the plugin, insert the trigger text %s into an existing page. The trigger will be auomatically replaced with a complete site wide table of content. You may also use the <code>create page</code> button below.'
		, 'tocc'),  ('<strong>' . htmlspecialchars(constant('TOCC_TAG')). '</strong>'));

	echo '</p>' . PHP_EOL 
		. '<h4>' . __('Click on the menu title to expand the option section then click on the option titles to get help!', 'tocc') . '</h4>' . PHP_EOL;

	tocc_option_header(__('General Options' ,'tocc'), 'tocc_general' , '');
	tocc_option(__('Table of Contents Title', 'tocc'), __(
'This text entered here will be displayed at the top of the table of content page. Leave blank to display no title. Include any required formatting tags.'
		, 'tocc'), 'tocc_toc_title', 'text', '40');
	tocc_option(__('Icon Theme', 'tocc'), __(
'Select the icon theme used to display next to each table of contents entry. Icon themes are a set of icons that are related in look and style. Icons are used to differentiate pages, links, blog categories, blog entries and form comments. Select <em>none</em> to inhibit the displaying of icons.'
		, 'tocc'), 'tocc_icon_set', 'select', explode(",", constant('TOCC_ICON_THEMES')), 0, "");
	tocc_option(__('Date Format', 'tocc'), __(
'Used to set the date display format when showing blog or forum comment dates. See <a href="http://php.net/manual/en/function.date.php" target="_none">http://php.net/manual/en/function.date.php</a> for format string details.'
		, 'tocc'), 'tocc_date_format', 'text', '40');
	tocc_option(__('Show Separate Lists', 'tocc'), __(
'Displays the page list, blog posts and forum comments as separate lists under a configurable title. See the settings below for <em>Page Title</em>, <em>Blog Title</em> and <em>Forum Comments Title</em>. The title strings may contain HTML formatting tags.'
		, 'tocc'), 'tocc_separate_lists', 'checkbox');	
	tocc_option(__('Show All Entries as List Items', 'tocc'), __(
'Displays all entries as unordered list items. If deselected, only page links, blog posts and forum comments will be displayed as list items. This option will be ignored if an icon theme is selected. This option was included for easier intergation into some existing themes.'
		, 'tocc'), 'tocc_all_listitems', 'checkbox');	
	tocc_option(__('More Prompt', 'tocc'), __(
'If enabled, the <em>more prompt</em> is displayed at the bottom of the blog posts and forum comments lists if not all posts or comments are displayed. clicking the link will display a new page showing all available posts or comments. This enables a smaller subset of the posts and/or comments to be displayed on the main site-map allowing greater readability.'
		, 'tocc'), 'tocc_more_prompt', 'text', '40');
	tocc_option(__('Show Author Credit', 'tocc'), __(
'Displays a small unobtrusive acknowledgement at the bottom of the table of contents. Please consider leaving the acknowledgement enabled as a token of appreciation for the hours spent developing this plugin.'
		, 'tocc'), 'tocc_show_ack', 'checkbox');
	tocc_option_end();


	tocc_option_header(__('Static Page Options' ,'tocc'), 'tocc_static_page');
	tocc_option(__('Show Static Pages', 'tocc'), __(
'Show static pages within the table of contents. Note that the pages will be sorted in menu order. Use the <strong>order</strong> attribute of each page (settable via page edit) to set the menu order.'
		, 'tocc'), 'tocc_show_static', 'checkbox');
	tocc_option(__('Page Title', 'tocc'), __(
'This text is used as the page list heading. The page list heading is only displayed if the separate page, blog post and forum comment lists option is checked. See <em>General Settings</em> for more information.'
		, 'tocc'), 'tocc_page_title', 'text', '40');
	tocc_option(__('Page Depth', 'tocc'), __(
'Determines the page depth to display. For example if set to 1, all parent pages will be shown and the immediate children of each parent page will also be shown. Set to 0 to show parent pages only.'
		, 'tocc'), 'tocc_page_depth', 'text', '5');
	tocc_option(__('Link Class', 'tocc'), __(
'If set, any &lt;a&gt; tags assigned to this class will be shown as a list item under the page in which the link was found. To use, add class="<em>linkclass</em>" to each &lt;a&gt; tag you wish to display in the table of contents. Leave blank to disable this option.'
		, 'tocc'), 'tocc_link_class', 'text', '40');
	tocc_option(__('Exclude Pages', 'tocc'), __(
'Select one or more pages to exclude from the table of content. By default, all pages are enabled.'
		, 'tocc'), 'tocc_page_exclude', 'checkbox', get_pages('depth=-1&sort_column=menu_order'));
	tocc_option_end();


	$no_blog = !is_blog_installed();
	tocc_option_header(__('Blog Post Options' ,'tocc'), 'tocc_blog_post');
	tocc_option(__('Show Blog Posts', 'tocc'), __(
'Show blog posts within the table of contents. Blog entries will automatically be separated by category.'
		, 'tocc'), 'tocc_show_blog', 'checkbox', '', $no_blog);
	tocc_option(__('Blog Title', 'tocc'), __(
'This is a mock page name used for sites that do not have a static post page. See the <em>Wordpress Settings/Reading</em> option for more details. The blog title is also used if static pages are not displayed or page lists, blog posts and forum comments are separately displayed.'
		, 'tocc'), 'tocc_blog_title', 'text', '40', $no_blog);
	tocc_option(__('Show Posts Total', 'tocc'), __(
'Displays the total number of posts in brackets for each category next to each category listing.'
		, 'tocc'), 'tocc_blog_info', 'checkbox', '', $no_blog);
	tocc_option(__('Number of Blog Posts', 'tocc'), __(
'Determines the maximum number of blog posts to display in each category. Set to 0 to display all posts.'
		, 'tocc'), 'tocc_blog_count', 'text', '5', $no_blog);
	tocc_option(__('Show More Prompt', 'tocc'), __(
'Displays a link at the bottom of the posts list to display more posts. The link will only be displayed if the current number of posts is limited (see the option above). The link text is set in the <em>general options</em> section.'
		, 'tocc'), 'tocc_blog_more', 'checkbox', '', $no_blog);
	tocc_option(__('Show Posts Warning', 'tocc'), __(
'Displays a message after the blog page name to alert the reader that only the latest <em>n</em> posts are shown. This text is not shown if all posts are displayed (by entering 0 in the "Number of Blog Entries" setting below).'
		, 'tocc'), 'tocc_blog_warn', 'checkbox', '', $no_blog);
	tocc_option(__('Posts Warning Text', 'tocc'), sprintf(__(
'The message to display if the post warning is enabled. Use %s as a place holder for the number of blog entries being displayed.'
		, 'tocc'), ('<code>' . constant('TOCC_NUMBER_SENTINEL') . '</code>')), 'tocc_blog_warntext', 'text', '40', $no_blog);
	tocc_option(__('Show Post Author', 'tocc'), __(
'Displays the author of the post.'
		, 'tocc'), 'tocc_blog_author', 'checkbox', '', $no_blog);
	tocc_option(__('Post Author Prefix', 'tocc'), __(
'The text to be displayed before the authors name.'
		, 'tocc'), 'tocc_blog_authorprefix', 'text', '5', $no_blog);
	tocc_option(__('Show Post Date', 'tocc'), __(
'Displays the creation date of the post.'
		, 'tocc'), 'tocc_blog_date', 'checkbox', '', $no_blog);
	tocc_option(__('Exclude Categories', 'tocc'), __(
'Select one or more categories to exclude from the table of content. By default, all categories are enabled.'
		, 'tocc'), 'tocc_blog_exclude', 'checkbox', $categories = get_categories(), $no_blog, 'cat_name');
	tocc_option_end();


	$no_forum = !function_exists('latest_activity');
	tocc_option_header(__('Forum Comment Options' ,'tocc'), 'tocc_forum_comment');
	echo '<p>' . __(
'Note that currently this option only works with <em>Forum Server</em> and <em>WP-Forum</em>.'
			, 'tocc') . '<br />&nbsp;<br /></p>';
	tocc_option(__('Show Forum Comments', 'tocc'), __(
'Show forum comments within the table of contents. The latest forum comments will be shown first. Note that currently this option only works with <em>Forum Server</em> and <em>WP-Forum</em>.'
		, 'tocc'), 'tocc_show_forum', 'checkbox', '', $no_forum);
	tocc_option(__('Forum Title', 'tocc'), __(
'This text is used as the forum list heading. The forum list heading is only displayed if the option is checked to separate the page, blog post and forum comment lists. See <em>General Settings</em> for more information.'
		, 'tocc'), 'tocc_forum_title', 'text', '40', $no_forum);
		tocc_option(__('Forum Page Name', 'tocc'), __(
'This is the name of the WordPress page that is used as a place holder to display the forum. The page name is required so that the plugin can display the forum comments underneath the correct page name. The forum page typically consists only of a tag such as &lt;!--VASTHTML--&gt;.'
		, 'tocc'), 'tocc_forum_page', 'select', get_pages('depth=-1&sort_column=menu_order'), $no_forum);
	tocc_option(__('Number of Forum Comments', 'tocc'), __(
'Determines the maximum number of forum comments to display. The latest comments will be displayed first. Set to 0 or less to display all comments (not recommended).'
		, 'tocc'), 'tocc_forum_count', 'text', '5', $no_forum);
	tocc_option(__('Show More Prompt', 'tocc'), __(
'Displays a link at the bottom of the comments list to display more comments. The link will only be displayed if the current number of comments is limited (see the option above). The link text is set in the <em>general options</em> section.'
		, 'tocc'), 'tocc_forum_more', 'checkbox', '', $no_forum);
	tocc_option(__('Show Comments Warning', 'tocc'), __(
'Displays a message after the forum page name to alert the reader that only the latest <em>n</em> comments are shown. This text is not shown if all comments are displayed (by entering 0 in the "Number of Forum Comments" setting below).'
		, 'tocc'), 'tocc_forum_warn', 'checkbox', '', $no_forum);
	tocc_option(__('Comment Warning Text', 'tocc'), sprintf(__(
'The message to display if the comment warning is enabled. Use %s as a place holder for the number of comment entries being displayed.'
		, 'tocc'), ('<code>' . constant('TOCC_NUMBER_SENTINEL') . '</code>')), 'tocc_forum_warntext', 'text', '40', $no_forum);
	tocc_option(__('Show Comment Author', 'tocc'), __(
'Displays the author of the comment.'
		, 'tocc'), 'tocc_forum_author', 'checkbox', '', $no_forum);
	tocc_option(__('Comment Author Prefix', 'tocc'), __(
'The text to be displayed before the authors name.'
		, 'tocc'), 'tocc_forum_authorprefix', 'text', '5', $no_forum);
	tocc_option(__('Show Comment Date', 'tocc'), __(
'Displays the creation date of the forum comment.'
		, 'tocc'), 'tocc_forum_date', 'checkbox', '', $no_forum);
	tocc_option_end();

			
	echo PHP_EOL . '<h4>' . __('Update Options', 'tocc') . '</h4><p>' . __(
'In order to save any changes, you must press the <em>Update Options</em> button before leaving this page or using the create page facility below.'
		, 'tocc') 
		. '</p>' . PHP_EOL 
		. '<input type="submit" name="info_update" value="' . __('Update Options', 'tocc') . ' &raquo;" />'
		. '&nbsp;&nbsp'
		. '<input type="submit" style="background-Color:#E0FFFF;"'
		. 'onmouseover="this.style.backgroundColor=\'red\'" '
		. 'onmouseout="this.style.backgroundColor=\'#E0FFFF\'"'
		. 'name="set_defaults" value="' . __('Load Default Options', 'tocc') . ' &raquo;" />'
		. PHP_EOL;

	echo PHP_EOL . '<h4>' . __('Create Page', 'tocc') . '</h4><p>' . __(
'For your convenience, the plugin can also create a new table of content page for you. Simply fill in the title and slug (path) details and press the <em>Create Page</em> button to create the page. The trigger text will be added automatically to the new page.'
		, 'tocc')
		. '</p>'
		. '<p><strong>' . __('Please note:', 'tocc') . ' </strong> ' . __(
'If you have changed any of the default settings, make sure you have pressed the <em>Update Options</em> button above before creating the new page.'
		, 'tocc') 
		. '</p>' . PHP_EOL;
	tocc_option(__('Page Title', 'tocc'), __(
'The table of content page name. The newly created page will be called this name. Ensure that there are no current pages of the same name.'
		, 'tocc'), 'tocc_toc_page', 'text', '40');
	tocc_option(__('Page Slug', 'tocc'), __(
'The page slug is used to determine the address of the page. It is typically the same name as the page name, <strong>however</strong> the page slug cannot contain spaces. It is common practice to replace a space with a dash.'
		, 'tocc'), 'tocc_toc_slug', 'text', '40');
	echo '<input type="submit" name="create_page" value="' . __('Create Page', 'tocc') . ' &raquo" />'
		. PHP_EOL;
		
	echo PHP_EOL . '<h4>' . __('Acknowledgments', 'tocc') . '</h4><p>' . __(
'I wish to personally acknowledge the following people for their valuable contributions:'
		, 'tocc') . '</p>' . PHP_EOL
		. '<div class="inside">'
		. '<a href="http://www.carlopoliti.net/">Carlo Politi</a> - ' . __('Italian translation', 'tocc')
		. '</div>' . PHP_EOL;

	echo '</form></div></div>' . PHP_EOL;

}
?>
